import { DiscoverLinks } from "./lib/discover";
import { Queue } from "./lib/queue";
import utils from "./lib/utils";
import _ from "lodash";

/**
 * 处理配置文件
 * @params config {config} 分析的配置文件
 */
const dealConfig = (config) => {
    config.blackPathList = config.blackPathList || [];
    config.blackPathList = config.blackPathList.map((path) => {
        return new RegExp(utils.replaceRegexp(path.regexp), path.scope);
    });
    // 查找enable是true的白名单
    config.whitePathList = _.filter(config.whitePathList, (list) => {
        return list.enable === true;
    });
    config.whitePathList = config.whitePathList.map((path) => {
        return new RegExp(utils.replaceRegexp(path.regexp), path.scope);
    });

    return config;
};

/**
 * 通过分析结果中得出的url处理分析出需要的
 * @params options 配置信息
 */
export const queueResultUrl = (options) => {
    return async(ctx, next) => {
        let config = dealConfig(ctx.config.urlAnalysis.queue);
        let queue = new Queue(config || {});
        let allowUrls = [];

        if (ctx.queueItem.analysisResultUrls) {
            ctx.queueItem.analysisResultUrls.forEach((url) => {
                allowUrls.push(queue.queueURL(url, ctx.queueItem.url ? ctx.queueItem : null));
            });

            ctx.queueItem.analysisResultUrlResult = allowUrls.filter((url) => {
                return url !== false;
            });
        }

        await next();
    };
};

/**
 * 分析得出的url处理分析出需要的
 * @params options 配置信息
 */
export default (options) => {
    return async(ctx, next) => {
        let discover = new DiscoverLinks(dealConfig(ctx.config.urlAnalysis.discover || {}));
        let queue = new Queue(dealConfig(ctx.config.urlAnalysis.queue || {}));
        let urls = await discover.discoverResources(ctx.queueItem);
        let allowUrls = [];

        urls.forEach((url) => {
            allowUrls.push(queue.queueURL(url, ctx.queueItem));
        });

        ctx.queueItem.analysisUrlResult = allowUrls.filter((url) => {
            return url !== false;
        });

        ctx.body.htmlAnalysis = true;

        await next();
    };
};